#ifndef INC_GTAPPDISP_H
#define INC_GTAPPDISP_H

#include "gtdisplay.h"

/************************************************************************\
*
* GtAppDisp class header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Size of hash table */
#define GTD_HASH_SIZE		128

class GtApp;
class GtWidget;

/* Type of selection request callback data */
typedef struct GtSelReqCD
{
	XEvent *xevent;
	GtWidget *w;
	Window req;
	Atom sel;
	Atom target;
	Atom prop;
	int fmt;
	ubyte *data;
	int len;
	sbool ok;
} GtSelReqCD;

/* Type of selection clear callback data */
typedef struct GtSelClrCD
{
	XEvent *xevent;
	GtWidget *w;
	Atom sel;
} GtSelClrCD;

/* Type of callback data for selection read */
typedef struct GtSelGetCD
{
	GtWidget *w;
	Atom sel;
	Atom type;
	int fmt;
	void *data;
	ulong len;
} GtSelGetCD;

/* Type of callback data for enter/leave widget callback */
typedef struct GtEnterLeaveCD
{
	GtWidget *wid;
	ulong state;
	sbool enter;
} GtEnterLeaveCD;

/* Index of standard atoms */
#define GTDSA_TARGETS		0
#define GTDSA_TIMESTAMP	1
#define GTDSA_MULTIPLE	2
#define GTDSA_MAX_NR		3

class GtAppDisp : public GtDisplay
{
public:
		/* Standard */
	GtAppDisp(void);
	void Destroy(void);
	sbool Init(GtApp *a,GtTag *tgs);
	sbool Init(GtTag *tgs);
	sbool Set(GtTag *tgs);
	void Get(GtTag *tgs);
	char *GetClass(void) { return("GtAppDisp"); }
	sbool IsSubClassOf(char *cn);
		/* Events */
	void HandleEvent(XEvent *xe);
		/* Linking */
	void SetRoot(GtWidget *o) { root=o; }
	GtWidget *GetRoot(void) { return(root); }
	void AddWidget(GtWidget *w);
	void RemWidget(GtWidget *w);
		/* Current widget for handle */
	GtWidget *GetCurrentWidget(void) { return(curr); }
		/* Focus and Grab */
	void SetFocus(GtWidget *o);
	void ClrFocus(GtWidget *o);
	void ChkFocus(GtWidget *o);
	void SetGrab(GtWidget *o);
	void ClrGrab(GtWidget *o);
	sbool HasFocus(GtWidget *o) { return(focus==o ? TRUE : FALSE); }
	sbool HasGrab(GtWidget *o) { return(grab==o ? TRUE : FALSE); }
		/* Selection management */
	sbool OwnSelection(GtWidget *w,Atom sel,Time t,GtCallB conv,GtCallB lose,
										 GtCallB done,GtUDPtr ud);
	void DisOwnSelection(GtWidget *w,Atom sel);
	void GetSelection(GtWidget *w,Atom sel,Atom target,Atom dest,Time tm,
										GtCallB cb,GtUDPtr ud);
	Atom *GetStdAtoms(void) { return(stdat); }
		/* Callbacks */
	void AddCB(ulong tag,GtCallB cb,GtUDPtr ud);
	void RemCB(ulong tag,GtCallB cb);
	void EnterLeave(GtWidget *w,sbool enter,ulong state);
		/* Values */
	inline long GetMCTime(void) { return(mclkt); }
	inline XrmDatabase GetDB(void) { return(dspdb); }
private:
	void HandleSelReq(XEvent *xe);
	void HandleSelClr(XEvent *xe);
	void HandleSelGet(XEvent *xe);
	void HandlePropDel(XEvent *xe);
	sbool AddSelReq(struct GtSelList *sl,Window req,Atom prop,Atom target);
	void SafeChange(Window req,Atom prop,Atom target,int fmt,
									int mode,ubyte *data,int len);
protected:
	GtSLink hash[GTD_HASH_SIZE];
	GtSLink sels,reqs,gets;
	GtCBList precb,entercb;
	GtApp *app;
	GtWidget *root,*focus,*grab,*curr;
	GtWidget *sel;
	Atom satom,stdat[3];
	Time stime;
	XrmDatabase dspdb;
	long mclkt;
};

extern GtAppDisp *GtMkAppDisp(GtApp *a,GtTag *tgs);
extern GtAppDisp *GtMkAppDisp(GtApp *a,ulong tag,...);

#endif /* INC_GTAPPDISP_H */
